@@ -1,12 +1,15 @@
 # frozen_string_literal: true
-require_relative('../explicit_require')
 
-Bootsnap::ExplicitRequire.with_gems('msgpack') { require('msgpack') }
-Bootsnap::ExplicitRequire.from_rubylibdir('fileutils')
+require_relative("../explicit_require")
+
+Bootsnap::ExplicitRequire.with_gems("msgpack") { require("msgpack") }
 
 module Bootsnap
   module LoadPathCache
     class Store
+      VERSION_KEY = "__bootsnap_ruby_version__"
+      CURRENT_VERSION = "#{RUBY_REVISION}-#{RUBY_PLATFORM}".freeze # rubocop:disable Style/RedundantFreeze
+
       NestedTransactionError = Class.new(StandardError)
       SetOutsideTransactionNotAllowed = Class.new(StandardError)
 
@@ -23,10 +26,11 @@ module Bootsnap
 
       def fetch(key)
         raise(SetOutsideTransactionNotAllowed) unless @txn_mutex.owned?
+
         v = get(key)
         unless v
-          @dirty = true
           v = yield
+          mark_for_mutation!
           @data[key] = v
         end
         v
@@ -34,14 +38,16 @@ module Bootsnap
 
       def set(key, value)
         raise(SetOutsideTransactionNotAllowed) unless @txn_mutex.owned?
+
         if value != @data[key]
-          @dirty = true
+          mark_for_mutation!
           @data[key] = value
         end
       end
 
       def transaction
         raise(NestedTransactionError) if @txn_mutex.owned?
+
         @txn_mutex.synchronize do
           begin
             yield
@@ -53,6 +59,11 @@ module Bootsnap
 
       private
 
+      def mark_for_mutation!
+        @dirty = true
+        @data = @data.dup if @data.frozen?
+      end
+
       def commit_transaction
         if @dirty
           dump_data
@@ -62,15 +73,20 @@ module Bootsnap
 
       def load_data
         @data = begin
-          File.open(@store_path, encoding: Encoding::BINARY) do |io|
-            MessagePack.load(io)
+          data = File.open(@store_path, encoding: Encoding::BINARY) do |io|
+            MessagePack.load(io, freeze: true)
+          end
+          if data.is_a?(Hash) && data[VERSION_KEY] == CURRENT_VERSION
+            data
+          else
+            default_data
           end
         # handle malformed data due to upgrade incompatibility
         rescue Errno::ENOENT, MessagePack::MalformedFormatError, MessagePack::UnknownExtTypeError, EOFError
-          {}
+          default_data
         rescue ArgumentError => error
           if error.message =~ /negative array size/
-            {}
+            default_data
           else
             raise
           end
@@ -80,19 +96,38 @@ module Bootsnap
       def dump_data
         # Change contents atomically so other processes can't get invalid
         # caches if they read at an inopportune time.
-        tmp = "#{@store_path}.#{Process.pid}.#{(rand * 100000).to_i}.tmp"
-        FileUtils.mkpath(File.dirname(tmp))
+        tmp = "#{@store_path}.#{Process.pid}.#{(rand * 100_000).to_i}.tmp"
+        mkdir_p(File.dirname(tmp))
         exclusive_write = File::Constants::CREAT | File::Constants::EXCL | File::Constants::WRONLY
         # `encoding:` looks redundant wrt `binwrite`, but necessary on windows
         # because binary is part of mode.
         File.open(tmp, mode: exclusive_write, encoding: Encoding::BINARY) do |io|
-          MessagePack.dump(@data, io, freeze: true)
+          MessagePack.dump(@data, io)
         end
-        FileUtils.mv(tmp, @store_path)
+        File.rename(tmp, @store_path)
       rescue Errno::EEXIST
         retry
       rescue SystemCallError
       end
+
+      def default_data
+        {VERSION_KEY => CURRENT_VERSION}
+      end
+
+      def mkdir_p(path)
+        stack = []
+        until File.directory?(path)
+          stack.push path
+          path = File.dirname(path)
+        end
+        stack.reverse_each do |dir|
+          begin
+            Dir.mkdir(dir)
+          rescue SystemCallError
+            raise unless File.directory?(dir)
+          end
+        end
+      end
     end
   end
 end
