@@ -1,5 +1,6 @@
 # frozen_string_literal: true
-$LOAD_PATH.unshift(File.expand_path('../../lib', __FILE__))
+
+$LOAD_PATH.unshift(File.expand_path("../lib", __dir__))
 
 if defined? Warning
   if Warning.respond_to?(:[]=)
@@ -7,37 +8,41 @@ if defined? Warning
   end
 end
 
-require('bundler/setup')
-require('bootsnap')
-require('bootsnap/compile_cache/yaml')
-require('bootsnap/compile_cache/json')
+require("bundler/setup")
+require("bootsnap")
+require("bootsnap/compile_cache/yaml")
+require("bootsnap/compile_cache/json")
 
-require('tmpdir')
-require('fileutils')
+require("tmpdir")
+require("fileutils")
 
-require('minitest/autorun')
-require('mocha/minitest')
+require("minitest/autorun")
+require("mocha/minitest")
 
-cache_dir = File.expand_path('../../tmp/bootsnap/compile-cache', __FILE__)
+cache_dir = File.expand_path("../tmp/bootsnap/compile-cache", __dir__)
 Bootsnap::CompileCache.setup(cache_dir: cache_dir, iseq: true, yaml: false, json: false)
 
 if GC.respond_to?(:verify_compaction_references)
   # This method was added in Ruby 3.0.0. Calling it this way asks the GC to
   # move objects around, helping to find object movement bugs.
-  GC.verify_compaction_references(double_heap: true, toward: :empty)
+  begin
+    GC.verify_compaction_references(double_heap: true, toward: :empty)
+  rescue NotImplementedError
+    # some platforms do not support GC compaction
+  end
 end
 
 module TestHandler
-  def self.input_to_storage(_i, p)
-    'neato ' + p
+  def self.input_to_storage(_input, path)
+    "neato " + path
   end
 
-  def self.storage_to_output(d, _a)
-    d.upcase
+  def self.storage_to_output(data, _kwargs)
+    data.upcase
   end
 
-  def self.input_to_output(_d, _a)
-    raise('but why tho')
+  def self.input_to_output(_data, _kwargs)
+    raise("but why tho")
   end
 end
 
@@ -67,7 +72,7 @@ module MiniTest
             hash ^= fnv1a_64(args_key)
           end
 
-          hex = hash.to_s(16).rjust(16, '0')
+          hex = hash.to_s(16).rjust(16, "0")
           "#{dir}/#{hex[0..1]}/#{hex[2..-1]}"
         end
 
@@ -80,10 +85,10 @@ module MiniTest
           hash
         end
 
-        def set_file(path, contents, mtime)
+        def set_file(path, contents, mtime = nil)
           FileUtils.mkdir_p(File.dirname(path))
           File.write(path, contents)
-          FileUtils.touch(path, mtime: mtime)
+          FileUtils.touch(path, mtime: mtime) if mtime
           path
         end
       end
@@ -95,7 +100,7 @@ module TmpdirHelper
   def setup
     super
     @prev_dir = Dir.pwd
-    @tmp_dir = Dir.mktmpdir('bootsnap-test')
+    @tmp_dir = Dir.mktmpdir("bootsnap-test")
     Dir.chdir(@tmp_dir)
     @prev = Bootsnap::CompileCache::ISeq.cache_dir
     Bootsnap::CompileCache::ISeq.cache_dir = @tmp_dir
